#include "prerequisites.h"
#include "TestDataGenerator.h"

namespace
{    
    bool Decide(float chance, int sampleSize = 127)
    {
        return csprng::Gen(sampleSize) < sampleSize * chance;
    }
}

namespace test
{
namespace data
{
    std::string NameGenerator::GenerateName()
    {
        if(m_nameTemplates.size() == 0)
        {
            throw std::exception("no name templates available.");
        }

        NameTemplate nameTemplate = m_nameTemplates[csprng::Gen(m_nameTemplates.size())];

        std::string generatedName;

        for(std::vector<std::string> phonemes : nameTemplate)
        {
            generatedName += phonemes[csprng::Gen(phonemes.size())];
        }

        return util::capitalise(generatedName);
    }   

    void NameGenerator::AddNameTemplate(NameTemplate nameTemplate)
    {
        m_nameTemplates.push_back(nameTemplate);
    }

    void NameGenerator::ClearNameTemplates()
    {
        m_nameTemplates.clear();
    }

    void NameGenerator::PushTemplateStack()
    {
        m_nameTemplateStack.push_back(m_nameTemplates);
        m_nameTemplates.clear();
    }

    void NameGenerator::PopTemplateStack()
    {
        if(m_nameTemplateStack.empty())
        {
            throw std::exception("no name templates available");
        }
        m_nameTemplates = m_nameTemplateStack.back();
        m_nameTemplateStack.pop_back();
    }

    void NameGenerator::AddExampleNameTemplates()
    {
        m_nameTemplates.push_back(NameTemplate { phonemes::Vowel, phonemes::Consonant, phonemes::Vowel });
        m_nameTemplates.push_back(NameTemplate { phonemes::Vowel, phonemes::MidConsonant, phonemes::Vowel });
        m_nameTemplates.push_back(NameTemplate { phonemes::Vowel, phonemes::DoubleConsonant, phonemes::Vowel });
        m_nameTemplates.push_back(NameTemplate { phonemes::Vowel, phonemes::MidConsonant, phonemes::DoubleVowel });
        m_nameTemplates.push_back(NameTemplate { phonemes::Fricative, phonemes::DoubleVowel, phonemes::Nasal });
        m_nameTemplates.push_back(NameTemplate { phonemes::Fricative, phonemes::DoubleVowel, phonemes::Consonant });
        m_nameTemplates.push_back(NameTemplate { phonemes::Fricative, phonemes::Vowel, phonemes::Consonant, phonemes::Vowel });
        m_nameTemplates.push_back(NameTemplate { phonemes::Fricative, phonemes::Vowel, phonemes::Consonant, phonemes::DoubleVowel });
        m_nameTemplates.push_back(NameTemplate { phonemes::Fricative, phonemes::Vowel, phonemes::DoubleGlide, phonemes::Vowel, phonemes::EndingNasal });
        m_nameTemplates.push_back(NameTemplate { phonemes::Fricative, phonemes::Vowel, phonemes::Alveolar, phonemes::Vowel, phonemes::EndingNasal });
        m_nameTemplates.push_back(NameTemplate { phonemes::Consonant, phonemes::Vowel, phonemes::Consonant, phonemes::Vowel, phonemes::Consonant });
        m_nameTemplates.push_back(NameTemplate { phonemes::Bilabial, phonemes::Vowel, phonemes::Consonant, phonemes::Vowel, phonemes::Consonant });
        m_nameTemplates.push_back(NameTemplate { phonemes::BilabialStop, phonemes::Liquid, phonemes::Vowel, phonemes::MidConsonant, phonemes::Vowel });
        m_nameTemplates.push_back(NameTemplate { phonemes::BilabialStop, phonemes::Liquid, phonemes::Vowel, phonemes::DoubleConsonant });
        m_nameTemplates.push_back(NameTemplate { phonemes::Consonant, phonemes::Vowel, phonemes::MidConsonant });
        m_nameTemplates.push_back(NameTemplate { phonemes::Consonant, phonemes::Vowel, phonemes::MidConsonant, phonemes::Vowel, phonemes::Consonant });
        m_nameTemplates.push_back(NameTemplate { phonemes::Consonant, phonemes::Vowel, phonemes::DoubleConsonant, phonemes::Vowel, phonemes::Consonant });
        m_nameTemplates.push_back(NameTemplate { phonemes::Nasal, phonemes::Vowel, phonemes::Fricative, phonemes::Vowel, phonemes::EndingNasal });
        m_nameTemplates.push_back(NameTemplate { phonemes::Consonant, phonemes::Vowel, phonemes::DoubleConsonant, phonemes::Liquid, phonemes::Vowel });
        m_nameTemplates.push_back(NameTemplate { phonemes::Consonant, phonemes::Vowel, phonemes::DoubleConsonant, phonemes::Vowel, phonemes::Consonant, phonemes::Vowel, phonemes::EndingNasal });
        m_nameTemplates.push_back(NameTemplate { phonemes::Consonant, phonemes::Vowel, phonemes::Palatal, phonemes::Vowel, phonemes::Consonant  });
    }

    std::string PlaceNameGenerator::Generate1stLine()
    {
        int sampleSize = 100;
        float chanceOfHavingALetter = 0.2;
        float chanceOfHavingAPrefix = 0.5;
        float chanceOfBeingAFlat = 0.5;

        bool isAFlat = Decide(chanceOfBeingAFlat);
        test::data::NameGenerator nameGenerator;
        nameGenerator.AddExampleNameTemplates();
        std::string name = util::capitalise(nameGenerator.GenerateName());

        switch(csprng::Gen(2))
        {
        case 0:
        default:
        {
            // e.g. 23, 72b
            std::string number = util::toStr(csprng::Gen(110));
            std::string letter(1, 'a' + char(csprng::Gen(13)));

            std::string areaType = util::capitalise(address::AreaTypes[csprng::Gen(address::AreaTypes.size())]);

            if(isAFlat)
            {
                number = "Flat " + number;
            }

            if(Decide(chanceOfHavingALetter))
            {
                return number + letter + " " + name + " " + areaType;
            }

            if(isAFlat)
            {
                return number;
            }

            return  number + " " + name + " " + areaType;
        }
        case 1:
        {
            // e.g. the <random name>
            std::string prefix = "The";

            std::string placeType = util::capitalise(address::PlaceTypes[csprng::Gen(address::PlaceTypes.size())]);


            if(Decide(chanceOfHavingAPrefix))
            {
                return prefix + " " + name + " " + placeType;
            }

            return name + " " + placeType;
        }
        }
    }

    std::string PlaceNameGenerator::Generate2ndLine()
    {
        test::data::NameGenerator nameGenerator;
        nameGenerator.AddExampleNameTemplates();

        std::string name = nameGenerator.GenerateName();
        std::string placeType = util::capitalise(address::AreaTypes[csprng::Gen(address::AreaTypes.size())]);

        return name + " " + placeType;
    }

    std::string PlaceNameGenerator::Generate3rdLine()
    {
        return address::UKTowns[csprng::Gen(address::UKTowns.size())];
    }
}
}
