#pragma once

namespace csprng
{
#ifdef _WIN32
    // cryptographically secure win32 entropic sampler (as of 2015)
    // https://msdn.microsoft.com/en-us/library/sxtz2fa8.aspx
    static unsigned int GenerateRandomNumber()
    {
        unsigned int randomNumber = 0;
        while(randomNumber == 0)
        {
            rand_s(&randomNumber);
        }
        return randomNumber;
    }
#endif
#ifdef __linux__
    // cryptographically secure linux entropic sampler (as of 2015)
    // https://en.wikipedia.org/wiki//dev/random
    static unsigned int GenerateRandomNumber()
    {
        // don't use std::ifstream as it buffers input - this can
        // exhaust the entropy pool prematurely
        FILE* randomDataSource = fopen("/dev/random", "r");
        if(randomDataSource == nullptr)
        {
            throw std::exception("unable to open /dev/random for reading.");
        }

        unsigned int randomData;
        unsigned int result = 0;
        while(result != 1)
        {
            result = fread(
                &randomData, sizeof(randomData), 1, randomDataSource);
        }

        fclose(randomDataSource);
        return randomData;
    }
#endif
    // generates a random number < max
    static unsigned int Gen(unsigned int max)
    {
        return GenerateRandomNumber() % max;
    }

    static std::string GenerateSalt(unsigned int length) 
    {
        const std::string alphanum =
            "0123456789"
            "abcdefghijklmnopqrstuvwxyz"
            "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

        std::string resultStr;
        resultStr.reserve(length);

        while(length-- > 0)
        {
            unsigned int sampleOffset = 
                GenerateRandomNumber() % (alphanum.length() - 1);

            resultStr.push_back(alphanum[sampleOffset]);
        }

        return resultStr;
    }
}